@@ -2,7 +2,7 @@
  *            the pppd from the command line.
  *            C. Scott Ananian <cananian@alumni.princeton.edu>
  *
- * $Id: pptp.c,v 1.36 2003/12/01 01:01:55 quozl Exp $
+ * $Id: pptp.c,v 1.40 2004/06/10 07:11:52 quozl Exp $
  */
 
 #include <sys/types.h>
@@ -32,6 +32,7 @@
 #include <setjmp.h>
 #include <errno.h>
 #include <sys/wait.h>
+#include <sys/param.h>
 #if defined(__APPLE__)
 #include "getopt.h"
 #else
@@ -53,9 +54,10 @@
 
 int syncppp = 0;
 int log_level = 1;
+int disable_buffer = 0;
 
 struct in_addr get_ip_address(char *name);
-int open_callmgr(struct in_addr inetaddr, char *phonenr, int argc,char **argv,char **envp, int pty_fd);
+int open_callmgr(struct in_addr inetaddr, char *phonenr, int argc,char **argv,char **envp, int pty_fd, int gre_fd);
 void launch_callmgr(struct in_addr inetaddr, char *phonenr, int argc,char **argv,char **envp);
 int get_call_id(int sock, pid_t gre, pid_t pppd, 
 		 u_int16_t *call_id, u_int16_t *peer_call_id);
@@ -80,6 +82,7 @@ void usage(char *progname)
             "  --debug		Run in foreground (for debugging with gdb)\n"
             "  --sync		Enable Synchronous HDLC (pppd must use it too)\n"
             "  --timeout <secs>	Time to wait for reordered packets (0.01 to 10 secs)\n"
+	    "  --nobuffer		Disable packet buffering and reordering completely\n"
             "  --logstring <name>	Use <name> instead of 'anon' in syslog messages\n"
             "  --localbind <addr>	Bind to specified IP address instead of wildcard\n"
             "  --loglevel <level>	Sets the debugging level (0=low, 1=default, 2=high)\n",
@@ -96,7 +99,7 @@ static int signaled = 0;
 void do_nothing(int sig)
 { 
     /* do nothing signal handler. Better than SIG_IGN. */
-    signaled = 1;
+    signaled = sig;
 }
 
 sigjmp_buf env;
@@ -163,6 +166,7 @@ int main(int argc, char **argv, char **envp)
             {"logstring", 1, 0, 0},
             {"localbind", 1, 0, 0},
             {"loglevel", 1, 0, 0},
+	    {"nobuffer", 0, 0, 0},
             {0, 0, 0, 0}
         };
         int option_index = 0;
@@ -213,7 +217,9 @@ int main(int argc, char **argv, char **envp)
                     log_level = atoi(optarg);
                     if (log_level < 0 || log_level > 2)
                         usage(argv[0]);
-                }
+                } else if (option_index == 9) { /* --nobuffer */
+		    disable_buffer = 1;
+		}
                 break;
             case '?': /* unrecognised option */
                 /* fall through */
@@ -255,6 +261,7 @@ int main(int argc, char **argv, char **envp)
 
         /* fork and wait. */
         signal(SIGUSR1, do_nothing); /* don't die */
+        signal(SIGCHLD, do_nothing); /* don't ignore SIGCHLD */
         parent_pid = getpid();
         switch (child_pid = fork()) {
             case -1:
@@ -274,6 +281,10 @@ int main(int argc, char **argv, char **envp)
                 if (!signaled) {
                     pause(); /* wait for the signal */
                 }
+ 
+                if (signaled == SIGCHLD)
+                    fatal("Child process died");
+ 
                 launch_pppd(ttydev, pppdargc, pppdargv); /* launch pppd */
                 perror("Error");
                 fatal("Could not launch pppd");
@@ -291,7 +302,7 @@ int main(int argc, char **argv, char **envp)
          * Open connection to call manager (Launch call manager if necessary.)
          */
         callmgr_sock = open_callmgr(inetaddr, phonenr, argc, argv, envp,
-		pty_fd);
+		pty_fd, gre_fd);
         /* Exchange PIDs, get call ID */
     } while (get_call_id(callmgr_sock, parent_pid, child_pid, 
                 &call_id, &peer_call_id) < 0);
@@ -329,7 +340,6 @@ shutdown:
         kill(parent_pid, SIGTERM);
     close(pty_fd);
     close(callmgr_sock);
-    sleep(3);     /* give ctrl manager a chance to exit */
     exit(0);
 }
 
@@ -354,7 +364,7 @@ struct in_addr get_ip_address(char *name)
 
 /*** start the call manager ***************************************************/
 int open_callmgr(struct in_addr inetaddr, char *phonenr, int argc, char **argv,
-        char **envp, int pty_fd)
+        char **envp, int pty_fd, int gre_fd)
 {
     /* Try to open unix domain socket to call manager. */
     struct sockaddr_un where;
@@ -381,8 +391,9 @@ int open_callmgr(struct in_addr inetaddr, char *phonenr, int argc, char **argv,
                 case 0: /* child */
                 {
                     close (fd);
-                    /* close the pty in the call manager */
+                    /* close the pty and gre in the call manager */
                     close(pty_fd);
+                    close(gre_fd);
                     launch_callmgr(inetaddr, phonenr, argc, argv, envp);
                 }
                 default: /* parent */
