@@ -2,7 +2,7 @@
  *                Handle the IP Protocol 47 portion of PPTP.
  *                C. Scott Ananian <cananian@alumni.princeton.edu>
  *
- * $Id: pptp_gre.c,v 1.34 2003/08/19 05:17:28 quozl Exp $
+ * $Id: pptp_gre.c,v 1.37 2004/06/11 01:40:56 quozl Exp $
  */
 
 #include <sys/types.h>
@@ -193,8 +193,22 @@ int decaps_hdlc(int fd, int (*cb)(int cl, void *pack, unsigned int len), int cl)
     /* start is start of packet.  end is end of buffer data */
     /*  this is the only blocking read we will allow */
     if ((end = read (fd, buffer, sizeof(buffer))) <= 0) {
-        warn("short read (%d): %s", end, strerror(errno));
-        if (errno == EIO) warn("pppd may have shutdown, see pppd log");
+        int saved_errno = errno;
+        warn("short read (%d): %s", end, strerror(saved_errno));
+	switch (saved_errno) {
+	  case EMSGSIZE: {
+	    int optval, optlen = sizeof(optval);
+	    warn("transmitted GRE packet triggered an ICMP destination unreachable, fragmentation needed, or exceeds the MTU of the network interface");
+#define IP_MTU 14
+	    if(getsockopt(fd, SOL_IP, IP_MTU, &optval, &optlen) < 0)
+	      warn("getsockopt: %s", strerror(errno));
+	    warn("getsockopt: IP_MTU: %d\n", optval);
+	    return 0;
+	  }
+	case EIO:
+    	    warn("pppd may have shutdown, see pppd log");
+	    break;
+	}
         return -1;
     }
     /* warn if the sync options of ppp and pptp don't match */
@@ -373,7 +387,7 @@ int decaps_gre (int fd, callback_t callback, int cl)
         first = 0;
         seq_recv = seq;
         return callback(cl, buffer + ip_len + headersize, payload_len);
-    /* out of order, check if the number is too low and discard thepacket. 
+    /* out of order, check if the number is too low and discard the packet. 
      * (handle sequence number wrap-around, and try to do it right) */
     } else if ( seq < seq_recv + 1 || WRAPPED(seq_recv, seq) ) {
 	if ( log_level >= 1 )
@@ -383,11 +397,18 @@ int decaps_gre (int fd, callback_t callback, int cl)
     /* sequence number too high, is it reasonably close? */
     } else if ( seq < seq_recv + MISSING_WINDOW ||
                 WRAPPED(seq, seq_recv + MISSING_WINDOW) ) {
+	stats.rx_buffered++;
         if ( log_level >= 1 )
-	    log("buffering packet %d (expecting %d, lost or reordered)", 
-		seq, seq_recv+1);
-        pqueue_add(seq, buffer + ip_len + headersize, payload_len);
-        stats.rx_buffered++;
+            log("%s packet %d (expecting %d, lost or reordered)",
+                disable_buffer ? "accepting" : "buffering",
+                seq, seq_recv+1);
+        if ( disable_buffer ) {
+            seq_recv = seq;
+            stats.rx_lost += seq - seq_recv - 1;
+            return callback(cl, buffer + ip_len + headersize, payload_len);
+        } else {
+            pqueue_add(seq, buffer + ip_len + headersize, payload_len);
+	}
     /* no, packet must be discarded */
     } else {
 	if ( log_level >= 1 )
